@@ -180,7 +180,7 @@ NumericVector select_line_connection(
   bool right = false;
   bool bottom = false;
 
-  if ((p1[0] >= b[0]) & (p1[0] <= b[2])) {
+  if ((p1[0] >= b[0]) && (p1[0] <= b[2])) {
     out[0] = p1[0];
   } else if (p1[0] > b[2]) {
     out[0] = b[2];
@@ -190,7 +190,7 @@ NumericVector select_line_connection(
     left = true;
   }
 
-  if ((p1[1] >= b[1]) & (p1[1] <= b[3])) {
+  if ((p1[1] >= b[1]) && (p1[1] <= b[3])) {
     out[1] = p1[1];
   } else if (p1[1] > b[3]) {
     out[1] = b[3];
@@ -622,7 +622,7 @@ Point spring_force(
 std::vector<double> rescale(std::vector<double> v) {
   double min_value = *std::min_element(v.begin(), v.end());
   double max_value = *std::max_element(v.begin(), v.end());
-  for (int i = 0; i < v.size(); i++) {
+  for (long unsigned int i = 0; i < v.size(); i++) {
     v[i] = (v[i] - min_value) / max_value;
   }
   return v;
@@ -675,6 +675,10 @@ DataFrame repel_boxes2(
     force_pull = 1e-6;
   }
 
+  if (force_push == 0) {
+    max_iter = 0;
+  }
+
   // Try to catch errors.
   if (n_texts > n_points) {
     Rcerr << "n_texts is " << n_texts << std::endl;
@@ -715,8 +719,8 @@ DataFrame repel_boxes2(
   ybounds.y = ylim[1];
 
   // Each data point gets a bounding circle.
-  std::vector<Point> Points(n_points, {0, 0});
-  std::vector<Circle> DataCircles(n_points, {0, 0, 0});
+  std::vector<Point> Points(n_points);
+  std::vector<Circle> DataCircles(n_points);
   for (int i = 0; i < n_points; i++) {
     DataCircles[i].x = data_points(i, 0);
     DataCircles[i].y = data_points(i, 1);
@@ -727,8 +731,8 @@ DataFrame repel_boxes2(
 
   // Add a tiny bit of jitter to each text box at the start.
   NumericVector r = rnorm(n_texts, 0, force_push);
-  std::vector<Box> TextBoxes(n_texts, {0, 0, 0, 0});
-  std::vector<Point> original_centroids(n_texts, {0, 0});
+  std::vector<Box> TextBoxes(n_texts);
+  std::vector<Point> original_centroids(n_texts);
   std::vector<double> TextBoxWidths(n_texts, 0);
   for (int i = 0; i < n_texts; i++) {
     TextBoxes[i].x1 = boxes(i, 0);
@@ -756,7 +760,11 @@ DataFrame repel_boxes2(
   // Rcout << std::endl;
 
   // Initialize velocities to zero
-  std::vector<Point> velocities(n_texts, {0, 0});
+  std::vector<Point> velocities(n_texts);
+  for (int i = 0; i < n_texts; i++) {
+    Point v = {0,0};
+    velocities[i] = v;
+  }
   double velocity_decay = 0.7;
 
   Point f, ci, cj;
